
# q-expansion of newform 1040.2.d.e, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1040
weight = 2
poly_data = [2, -4, 4, 2, 2, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[38, 2, -1, -4, 8, -1], 23], [[29, 10, -5, -20, 17, -5], 23], [[-13, 32, 30, 5, -10, 7], 23], [[27, -70, -11, -21, 19, -11], 23], [[26, -64, -37, -10, 20, -14], 23]]

hecke_ring_character_values = [[911, [1, 0, 0, 0, 0, 0]], [261, [1, 0, 0, 0, 0, 0]], [417, [-1, 0, 0, 0, 0, 0]], [561, [1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0], [0, 0, 0, -1, 1, 0], [0, 1, 0, 0, -1, 0], [0, 0, 0, -1, 1, 1], [0, 1, 0, 0, 0, 0], [0, 0, 0, -1, 0, 0], [0, 0, 0, -2, 0, 2], [-4, 1, -2, 0, 0, 0], [0, 0, 0, 5, -3, 2], [5, -1, 1, 0, 0, 0], [2, 3, -2, 0, 0, 0], [0, 0, 0, -1, -1, 3], [-2, 2, -2, 0, 0, 0], [0, 0, 0, 1, 1, 0], [0, 0, 0, -3, -1, 7], [0, 0, 0, 6, -2, 4], [2, -3, 4, 0, 0, 0], [-3, 1, -3, 0, 0, 0], [0, 0, 0, 5, -1, -1], [2, -3, -4, 0, 0, 0], [0, 0, 0, 3, 3, -5], [-10, 0, -2, 0, 0, 0], [0, 0, 0, 7, -3, -1], [6, 4, -4, 0, 0, 0], [0, 0, 0, 2, 0, -6], [2, 6, 0, 0, 0, 0], [0, 0, 0, -5, 5, 4], [0, 0, 0, -5, -5, 0], [6, 4, 2, 0, 0, 0], [0, 0, 0, 6, 4, 6], [0, 0, 0, -1, -3, -4], [6, -6, 10, 0, 0, 0], [0, 0, 0, -8, 6, -2], [-2, 4, 10, 0, 0, 0], [8, 2, 8, 0, 0, 0], [6, 3, 0, 0, 0, 0], [0, 0, 0, -10, 4, 4], [0, 0, 0, 1, 3, -3], [0, 0, 0, -9, -3, -5], [0, 0, 0, 2, 2, 4], [-4, -2, 4, 0, 0, 0], [-5, -3, 7, 0, 0, 0], [-8, 4, -4, 0, 0, 0], [0, 0, 0, 12, -14, 6], [0, 0, 0, -2, 0, 0], [4, -2, 8, 0, 0, 0], [12, 2, 8, 0, 0, 0], [0, 0, 0, 5, 7, -5], [0, 0, 0, -1, -3, -3], [10, -2, 12, 0, 0, 0], [0, 0, 0, 2, 2, -14], [2, 11, -6, 0, 0, 0], [0, 0, 4, 0, 0, 0], [-14, 2, -2, 0, 0, 0], [0, 0, 0, 2, -6, 6], [0, 0, 0, 13, -1, 0], [-10, 10, -12, 0, 0, 0], [4, -7, 10, 0, 0, 0], [0, 0, 0, 0, -4, -10], [0, -4, 12, 0, 0, 0], [0, 0, 0, -3, 5, -8], [0, 0, 0, 11, -1, -1], [0, 0, 0, 11, 9, -1], [8, -8, 0, 0, 0, 0], [0, 0, 0, 4, -2, 4], [0, 0, 0, -11, 17, -3], [12, -5, 0, 0, 0, 0], [0, 0, 0, -8, -6, 0], [0, 0, 0, 13, -1, -2], [10, -10, 4, 0, 0, 0], [0, 0, 0, 1, 13, 5], [4, -11, -14, 0, 0, 0], [0, 0, 0, -19, -9, -4], [0, 0, 0, 6, 8, 0], [10, 13, -6, 0, 0, 0], [0, 0, 0, 5, -1, 9], [-14, -8, 8, 0, 0, 0], [0, 0, 0, 15, -1, 7], [16, -2, -6, 0, 0, 0], [-18, -2, 6, 0, 0, 0], [2, -10, -10, 0, 0, 0], [-14, 0, 6, 0, 0, 0], [14, 3, 14, 0, 0, 0], [0, 0, 0, -8, 16, 0], [-2, -16, 6, 0, 0, 0], [0, 0, 0, -7, -9, -2], [-14, 6, -18, 0, 0, 0], [0, 0, 0, 10, 4, -6], [16, 0, -2, 0, 0, 0], [0, 0, 0, 17, -13, 1], [0, 0, 0, 9, -9, 16], [12, -11, 2, 0, 0, 0], [0, 0, 0, -17, -11, 3], [8, 2, 4, 0, 0, 0], [8, -7, -2, 0, 0, 0], [0, 0, 0, 1, 17, 4], [4, 12, 2, 0, 0, 0], [-17, 9, -17, 0, 0, 0], [0, 0, 0, 11, -15, 4], [-4, 16, -6, 0, 0, 0], [0, 0, 0, 27, -1, 0], [0, 0, 0, 3, -5, 19], [0, 0, 0, -11, 1, -4], [-7, -3, 1, 0, 0, 0], [-6, -4, -14, 0, 0, 0], [0, 0, 0, 33, 1, 1], [0, 0, 0, 19, 1, -5], [0, 0, 0, 10, -4, -2], [4, 20, -12, 0, 0, 0], [-14, -12, 0, 0, 0, 0], [0, 0, 0, 9, 9, -16], [0, 0, 0, -8, 10, -12], [0, 0, 0, -4, 10, 6], [6, 9, -4, 0, 0, 0], [4, 3, -8, 0, 0, 0], [18, -8, 4, 0, 0, 0], [0, 0, 0, 11, -3, 1], [0, 0, 0, -9, -11, 0], [0, 0, 0, -18, 0, 12], [-6, 10, -10, 0, 0, 0], [-8, -6, 8, 0, 0, 0], [0, 0, 0, -12, 6, 0], [0, 0, 0, -4, -16, -10], [0, 0, 0, -1, 1, 5], [-14, 9, -14, 0, 0, 0], [2, -18, -4, 0, 0, 0], [-28, 4, -10, 0, 0, 0], [-6, 2, 2, 0, 0, 0], [0, 0, 0, 1, 13, -18], [0, 0, 0, -10, -16, 12], [-6, -17, -2, 0, 0, 0], [0, 0, 0, -19, 27, -7], [14, -14, 14, 0, 0, 0], [0, 0, 0, 26, -2, -8], [8, -6, -6, 0, 0, 0], [28, -2, 6, 0, 0, 0], [0, 0, 0, 5, -7, 1], [0, 0, 0, 7, 5, 15], [0, 0, 0, -20, -10, 2], [11, -19, 11, 0, 0, 0], [-6, -11, -2, 0, 0, 0], [0, 10, -4, 0, 0, 0], [0, 0, 0, -19, 11, -6], [0, 0, 0, 29, 11, 5], [-25, 3, -1, 0, 0, 0], [4, 3, 24, 0, 0, 0], [0, 0, 0, 15, -9, -1], [0, 0, 0, -8, -12, -4], [4, -6, -12, 0, 0, 0], [0, 0, 0, 41, -1, -5], [0, 0, 0, 36, -10, 8], [-3, -7, -7, 0, 0, 0], [0, 0, 0, -25, -5, -10], [0, 0, 0, -45, -3, 6], [0, 0, 0, 13, 11, 12], [-12, 10, -8, 0, 0, 0], [-20, 0, 8, 0, 0, 0], [-12, -8, 8, 0, 0, 0], [0, 0, 0, -2, 6, -26], [6, 2, 0, 0, 0, 0], [0, 0, 0, -15, -1, 13], [0, 0, 0, 0, -8, 8], [0, 0, 0, -33, 13, -11], [4, 6, -16, 0, 0, 0], [0, 0, 0, -27, 5, 9], [0, 0, 0, -11, -17, -7], [-10, -32, 10, 0, 0, 0], [0, 0, 0, -24, 32, -14], [12, 10, -18, 0, 0, 0], [0, 0, 0, -14, 22, 4], [-12, -5, 16, 0, 0, 0], [-2, -2, 20, 0, 0, 0], [0, 6, 0, 0, 0, 0], [0, 0, 0, -47, 5, -3], [-22, 12, 14, 0, 0, 0], [-34, -12, 4, 0, 0, 0], [34, -7, -4, 0, 0, 0], [14, -18, 20, 0, 0, 0], [0, 0, 0, -21, -17, -4], [10, -16, -20, 0, 0, 0], [0, 0, 0, 1, -1, -25], [-4, -16, 20, 0, 0, 0], [0, 0, 0, 2, -6, -16], [0, 0, 0, 16, 6, 10], [0, 0, 0, 35, -15, 1], [31, 17, 3, 0, 0, 0], [0, 0, 0, 4, -2, -2], [0, 0, 0, 15, -31, 9], [28, 8, 16, 0, 0, 0], [2, -21, 16, 0, 0, 0], [0, 0, 0, 8, 14, 8], [0, 0, 0, -9, 9, -17], [52, -2, -4, 0, 0, 0], [-26, 4, 2, 0, 0, 0], [0, 0, 0, 13, 9, -12], [0, 0, 0, -8, -12, -4], [-16, -8, -16, 0, 0, 0], [0, 0, 0, -4, 14, -14], [0, 0, 0, -5, -25, -9], [0, 0, 0, 11, -21, 22], [2, -18, -4, 0, 0, 0], [2, 14, -2, 0, 0, 0], [0, 0, 0, 32, -34, 12], [31, -3, 19, 0, 0, 0], [10, 23, -14, 0, 0, 0], [0, 0, 0, 22, 14, -8], [-14, -9, -6, 0, 0, 0], [0, 0, 0, 1, -21, 22], [18, 4, -12, 0, 0, 0], [0, -2, -20, 0, 0, 0], [0, 0, 0, -8, 8, 12], [29, -19, 17, 0, 0, 0], [0, 0, 0, 19, -9, -6], [0, 0, 0, -9, 25, -23], [-10, 1, 28, 0, 0, 0], [-14, 10, -14, 0, 0, 0], [0, 0, 0, 5, 19, 20], [-26, 2, 4, 0, 0, 0], [0, 0, 0, -27, -9, 5], [0, 0, 0, 18, -12, -12], [-19, -23, 1, 0, 0, 0], [-12, -1, 8, 0, 0, 0], [-24, 2, -22, 0, 0, 0], [0, 0, 0, -1, -3, -1], [0, 0, 0, 9, 23, -16], [-13, -17, -17, 0, 0, 0], [0, 0, 0, -32, 16, -20], [-10, 22, -26, 0, 0, 0], [0, 0, 0, 11, 19, 8], [-40, 3, -6, 0, 0, 0], [0, 0, 0, 6, -12, 16], [8, -18, -8, 0, 0, 0], [28, 11, -6, 0, 0, 0], [2, -10, 0, 0, 0, 0], [0, 0, 0, 19, -15, 12], [0, 0, 0, -45, 9, -9], [30, -14, 18, 0, 0, 0], [0, 0, 0, 39, -25, 11], [8, -14, 24, 0, 0, 0], [14, 20, 2, 0, 0, 0], [0, 0, 0, -23, 19, -27], [16, 20, -20, 0, 0, 0], [0, 0, 0, -7, 1, 0], [16, -4, -22, 0, 0, 0], [0, 0, 0, 28, -14, 2], [-30, 12, -2, 0, 0, 0], [0, 0, 0, 13, -21, 3], [10, 3, -10, 0, 0, 0], [-20, 15, 0, 0, 0, 0], [0, 0, 0, 11, -7, 24], [0, 0, 0, 11, -5, -9], [40, -18, 6, 0, 0, 0], [0, 0, 0, 7, -11, 21], [-38, -12, 4, 0, 0, 0], [0, 0, 0, 22, 0, -4], [-20, 15, -10, 0, 0, 0], [-15, 3, 9, 0, 0, 0], [0, 0, 0, 25, 3, 21], [0, 0, 0, -40, -2, 10], [0, 0, 0, 53, -3, 5], [0, 0, 0, 7, 27, -12], [0, 0, 0, -43, 7, 10], [36, 4, -2, 0, 0, 0], [0, 0, 0, -36, -8, -2], [0, 0, 0, 19, 11, -9], [-4, -12, -8, 0, 0, 0], [-34, 10, -28, 0, 0, 0], [-18, -16, -4, 0, 0, 0], [0, 0, 0, 17, -17, 23], [0, 0, 0, -10, 14, 4], [-6, 22, -28, 0, 0, 0], [0, 0, 0, 3, -31, -13], [0, 0, 0, -4, -10, -18], [24, 34, -8, 0, 0, 0], [0, 0, 0, 6, -32, 22], [0, 0, 0, -35, -1, 9], [0, 0, 0, -13, -11, -7], [6, 30, -28, 0, 0, 0], [-18, 2, -22, 0, 0, 0], [14, -28, 38, 0, 0, 0], [0, 0, 0, 11, 19, 22], [-4, 19, 4, 0, 0, 0], [0, 0, 0, 11, 7, -10], [22, -10, 36, 0, 0, 0], [0, 0, 0, 7, 33, -1], [2, 2, -26, 0, 0, 0], [0, 0, 0, 26, -6, -10], [0, 0, 0, 24, -6, 20], [38, -17, 20, 0, 0, 0], [29, -11, 25, 0, 0, 0], [-15, 17, -7, 0, 0, 0], [0, 0, 0, 11, -25, 16], [0, 0, 0, -2, 20, 26], [-18, 27, 18, 0, 0, 0], [0, 0, 0, 14, -32, 24], [-14, 12, 32, 0, 0, 0], [-44, 14, -12, 0, 0, 0], [0, 0, 0, 0, 18, -6], [64, 0, 0, 0, 0, 0], [0, 0, 0, -15, -9, 9], [0, 0, 0, 34, -10, -10], [0, 0, 0, -44, 10, -12], [-24, -26, -4, 0, 0, 0]]
